// We need the Adafruit Arcada Library for our basic functionality
#include <Adafruit_Arcada.h>
#include <Adafruit_NeoPixel.h>
#include "lanes.h"
#include "images.h"
#include "colors.h"

#ifdef __AVR__
 #include <avr/power.h> // Required for 16 MHz Adafruit Trinket
#endif

// Global variables
#define PIN 8
#define NUMPIXELS 6
#define TOTAL 3
Adafruit_Arcada arcada;
// handle buttons
uint8_t buttonState = arcada.readButtons();

// timer for spawnObstacles
uint32_t previousMillis;
uint32_t currentMillis;

// high score
int highscore=0;
int score=0;

// music
File bgm;
bool musicPlaying = false;

// for picking up to recharge the powers in hard mode
typedef struct icePower{
  bool activated = false;
  int power = 5;
  int max = 10;
  int color = ARCADA_BLUE;
}icePower;

icePower icep;

typedef struct firePower{
  bool activated = false;
  int power = 5;
  int max = 10;
  int color = ARCADA_ORANGE;
}firePower;

firePower firep;

typedef struct pickUp{
  bool picked = false;
  int power = random(3);
  int lane = random(2);
  int size = 0;
  int type = random(1);
  int posX;
  int posY = 150;
  int color;
}pickUp;

const int max_PU = 3;
pickUp PUelements[max_PU];
pickUp pu0;
pickUp pu1;
pickUp pu2;


// obstacles
typedef struct Obstacle{
  bool chosen;
  int obstacleType;
  int power;
  int lane;
  int posX;
  int posY = 60;
  int size = 5;
  int color;
  int borderColor;
}Obstacle;

const int max_obstacles = 8;
Obstacle obstacles[max_obstacles];

Obstacle obst0;
Obstacle obst1;
Obstacle obst2;
Obstacle obst3;
Obstacle obst4;
Obstacle obst5;
Obstacle obst6;
Obstacle obst7;


typedef struct IceObject{
  int lane;
  int posX;
  int posY = 100;
  int sizeX = 8;
  int sizeY = 16;
}IceObject;

IceObject iceObj;


typedef struct FireObject{
  int lane;
  int posX;
  int posY;
  int sizeX;
  int sizeY;
const uint16_t* fire;
const uint8_t* fire_mask;
}FireObject;

FireObject fireObj0;
FireObject fireObj1;
FireObject fireObj2;
const int fire_states= 6;
FireObject fires[fire_states];
int fire_counter = 0;


typedef struct Cloud{
  int posX=20;
  int posY=25;
  bool toRight=true;
}Cloud;

Cloud c;

typedef struct Sun{
  int posX=10;
  int posY=10;
  int facePosX=18;
  int facePosY=22;
  int beamCounter=0;
  const uint16_t* sun=sunBase;
  const uint8_t* sunMask=sun_mask;
  const uint16_t* face=sunFace;
  const uint8_t* faceMask=sunFace_mask;
}Sun;

Sun s;

bool inMenu = true;
bool inSelection = false;
int level = 0;

bool buttonLeft;
bool buttonRight;

int lane = 1;
int x_pos = ARCADA_TFT_WIDTH/2;
int y_pos = ARCADA_TFT_HEIGHT-20;
int obstacleLane;

int lives = 3;
bool livesLost = false;
int blinking = 0;
bool jumping = false;
bool sliding = false;
bool icing = false;
bool firing = false;


const uint16_t* runner;
const uint8_t* runner_mask;
int leg_counter = 0;




void setup() {
  // put your setup code here, to run once:
  if (!arcada.arcadaBegin())
  {
    errorReporter("Fatal error: Arcada library failed.");
  }

  // initialize the file system on the flash (first flashing should be done via CircuitPython)
  if (!arcada.filesysBegin()) { // arcada.filesysBeginMSD() for "USB-Stick" mode
    errorReporter("Fatal error: Filesystem failed.");
  }

  // initialize serial connection
  Serial.begin(9600);
  Serial.println("Booting into game...");
  
  // configure board display and enable frame buffer mode
  arcada.displayBegin();
  arcada.setBacklight(255);
  arcada.createFrameBuffer(ARCADA_TFT_WIDTH, ARCADA_TFT_HEIGHT);

  Serial.println("Initializing display...");

  // we start our loop in the menu
  inMenu = true;
  
  Serial.println("Finished booting!");

  loadMusic();
  
  // set Obstacles
  obstacles[0] = obst0;
  obstacles[1] = obst1;
  obstacles[2] = obst2;
  obstacles[3] = obst3;
  obstacles[4] = obst4;
  obstacles[5] = obst5;
  obstacles[6] = obst6;
  obstacles[7] = obst7;
  if (level == 0) {
    for(int i=0; i<max_obstacles; i++) {
      obstacles[i].power = 1;
    }
  } else {
    for(int i=0; i<max_obstacles; i++) {
      obstacles[i].power = random(5);
    }
  }

  // objects to pick up for recharging power in hard mode
  PUelements[0] = pu0;
  PUelements[1] = pu1;
  PUelements[2] = pu2;

  // set entries for the fire animation
  fireObj0.sizeX = 8;
  fireObj0.sizeY = 11;
  fireObj0.posY = 91;
  fireObj0.fire = fire0;
  fireObj0.fire_mask = fire0_mask;
  fireObj1.sizeX = 8;
  fireObj1.sizeY = 18;
  fireObj1.posY = 88;
  fireObj1.fire = fire1;
  fireObj1.fire_mask = fire1_mask;
  fireObj2.sizeX = 8;
  fireObj2.sizeY = 26;
  fireObj2.posY = 80;
  fireObj2.fire = fire2;
  fireObj2.fire_mask = fire2_mask;
  
  fires[0] = fireObj0;
  fires[1] = fireObj1;
  fires[2] = fireObj2;
  fires[3] = fireObj2;
  fires[4] = fireObj2;
  fires[5] = fireObj0;

  previousMillis = millis();
}


/**
 * main game loop
 */
void loop() {
  // put your main code here, to run repeatedly:
  // handle buttons
  uint8_t buttonState = arcada.readButtons();
  
  GFXcanvas16 *canvas = arcada.getCanvas();
  canvas->fillScreen(ARCADA_WHITE);


  // let's just pretend it's a state machine
  if (inMenu) {
    handleMenu();
    arcada.enableSpeaker(false);
  } else if (inSelection){
    handleSelection();
    arcada.enableSpeaker(false);
  } else {
    arcada.enableSpeaker(true);
    laneSwitcher();
    delimitLanes();
    spawnObstacles();
    handleGameplay();
    initPU();
  }

  arcada.blitFrameBuffer(0, 0, true, false);
  yield();

}


/**
 * the start screen only seen once when opening the game
 */
void handleMenu() {
  uint8_t buttonState = arcada.readButtons();
  if (buttonState & ARCADA_BUTTONMASK_A) {
    inMenu = false;
    inSelection = true;
  }

  uint32_t sampleRate;
//  arcada.timerCallback(sampleRate, timedInterruptCallback);
  
  GFXcanvas16 *canvas = arcada.getCanvas();
  canvas->fillScreen(ARCADA_WHITE);
  
  canvas->setTextSize(2);
  canvas->setTextColor(ARCADA_BLACK);
  canvas->setCursor(15, 10);
  canvas->print("RUNNER GAME");

  canvas->setTextSize(1);
  canvas->setTextColor(ARCADA_BLACK);
  canvas->setCursor(13, 100);
  canvas->print("- press A to continue -");

  canvas->drawRGBBitmap(70,60, s.face, s.faceMask, 16,11);
  canvas->drawRGBBitmap(87,60, ice,ice_mask, 8,16);
  canvas->drawRGBBitmap(60,60, fire1,fire1_mask, 8,18);
}


/**
 * you can choose easy mode (level=0) and hard mode (level=1) in the selection screen
 */
void handleSelection() {
  uint8_t buttonState = arcada.readButtons();
  GFXcanvas16 *canvas = arcada.getCanvas();
  
  if (buttonState & ARCADA_BUTTONMASK_START) {
    inSelection = false;
  }

  
  canvas->setTextColor(ARCADA_BLACK);
  canvas->setTextSize(1);
  canvas->setCursor(20, 10);
  canvas->print("press START to start");
  
  canvas->setTextSize(2);
  canvas->setCursor(20, 30);
  canvas->print("easy");
  canvas->setCursor(20, 70);
  canvas->print("hard");


  if (buttonState & ARCADA_BUTTONMASK_DOWN) {
    level = 1;
  }
  if (buttonState & ARCADA_BUTTONMASK_UP) {
    level = 0;
  }
  
  if (level==0) {
    canvas->setTextColor(ARCADA_BLUE);
    canvas->setCursor(20, 30);
    canvas->print("easy");
    canvas->drawRGBBitmap(10,30, ice,ice_mask, 8,16);
  } else {
    canvas->setTextColor(ARCADA_BLUE);
    canvas->setCursor(20, 70);
    canvas->print("hard");
    canvas->setTextSize(1);
    canvas->setCursor(20, 88);
    canvas->print("Obstacles have power");
    canvas->setCursor(20,97);
    canvas->print("levels now, and can");
    canvas->setCursor(20,106);
    canvas->print("kill you directly.");
    canvas->drawRGBBitmap(10,70, fire1,fire1_mask, 8,18);
  }
}


/**
 * draws the background of the game
 */
void delimitLanes(){
  GFXcanvas16 *canvas = arcada.getCanvas();
  canvas->drawRGBBitmap(0,0 ,background, ARCADA_TFT_WIDTH,ARCADA_TFT_HEIGHT);
}


/** 
 *  puts the runner in the correct lane according to the input of the LEFT- and RIGHT-button
 */
void laneSwitcher(){
  // handle buttons
  uint8_t buttonState = arcada.readButtons();

  if(buttonState & ARCADA_BUTTONMASK_RIGHT){
    delay(50);
    lane = lane + 1;
    if(lane > 2){
      lane = 2;
    }
  }
  if(buttonState & ARCADA_BUTTONMASK_LEFT){
    delay(50);
    lane = lane - 1;
    if(lane < 0){
      lane = 0;
    }
  }
}





/**
 * returns the current y position of the runner when jumping
 */
int jump(){
  if (y_pos > ARCADA_TFT_HEIGHT-50) {
    y_pos = y_pos-9;
  } else {
    y_pos = ARCADA_TFT_HEIGHT-20;
    jumping = false;
  }
  return y_pos;
}


/**
 * returns the y position of the runner when sliding
 */
int slide(){
  if (y_pos <= ARCADA_TFT_HEIGHT-3) {
    y_pos = y_pos+4;
  } else {
    y_pos = ARCADA_TFT_HEIGHT-20;
    sliding = false;
  }
  return y_pos;
}


/**
 * returns the current x position of the runner
 */
int handleLane() {
  if (lane==0) {
    x_pos = ARCADA_TFT_WIDTH/2-52;
  }
  if (lane==1) {
    x_pos = ARCADA_TFT_WIDTH/2-3;
  }
  if (lane==2) {
    x_pos = ARCADA_TFT_WIDTH/2+48;
  }
  return x_pos;
}


/**
 * updates the values for drawing the obstacles in the correct coordinates
 */
void updateObstacles(){
  for (int i=0; i<max_obstacles; i++) {
    switch(obstacles[i].lane){
    case 0:
      obstacles[i].posX = obstacles[i].posX-6;
    break;
    case 1:
      obstacles[i].posX = obstacles[i].posX-1;
    break;
    case 2:
      obstacles[i].posX = obstacles[i].posX+4;
    break;
    }
    obstacles[i].posY = obstacles[i].posY+4;
    obstacles[i].size = obstacles[i].size+2;

    // Collision detection
    if(obstacles[i].chosen) {
      collisionDetection(obstacles[i]);
    }
  }
}


/**
 * detects if the runner has run into an obstacle and decreases 'live' accordingly
 */
void collisionDetection(Obstacle obs){
  if(obs.lane == lane){
    if(obs.posY+obs.size>=122 && obs.posY+obs.size<=125 && !livesLost && !jumping && !sliding) {
      if(obs.posY<=y_pos && obs.posY+obs.size>=y_pos) {
        blinking = 8; // for the runner animation
        if(level == 0){
          lives --;
          livesLost = true;
        } else {
          lives -= obs.power;
          livesLost = true;
        }
        
      }
    }
    if(obs.posY+obs.size>125) {
      livesLost = false;
    }
  }
}


/**
 * updates the power gauges in hard mode
 */
void powerUpdate(Obstacle obs, int type){
  if(obs.obstacleType == 1 && type == 1){
    firep.power += obs.power;
    if(firep.power > firep.max){
      firep.power = firep.max;
    }
  }
  if(obs.obstacleType == 0 && type == 0){
    icep.power += obs.power;
    if(icep.power > icep.max){
      icep.power = icep.max;
    }
  }
}

/**
 * function for handling the actual game
 */
void handleGameplay() {
  uint8_t buttonState = arcada.readButtons();
  
  int x=ARCADA_TFT_WIDTH/2;
  int y=ARCADA_TFT_HEIGHT-7;
  
  GFXcanvas16 *canvas = arcada.getCanvas();



  // animation for cloud and sun
  updateSunCloud();
  canvas->drawRGBBitmap(c.posX,c.posY, cloud,cloud_mask, 49,20);
  canvas->drawRGBBitmap(s.posX,s.posY, s.sun,s.sunMask, 32,32);
  canvas->drawRGBBitmap(s.facePosX,s.facePosY, s.face,s.faceMask, 16,11);

  handleButtons();



  // animation for runner
  if (!jumping && !sliding) {
    legs();
  }
  
  if (jumping) {
    canvas->drawRGBBitmap(handleLane(), jump(), runner,runner_mask, 11,20);
  } else if (sliding) {
    canvas->drawRGBBitmap(handleLane(), slide(), runner,runner_mask, 11,20);
  } else {
    if(blinking>0) {
      // animates the blinking effect if we were just hit by an obstacle
      justLostALife();
    } else {
      canvas->drawRGBBitmap(handleLane(),y_pos, runner, runner_mask, 11,20);
    }
  }

  // animation for the powers
  if (icing) {
    canvas->drawRGBBitmap(iceObj.posX,iceObj.posY ,ice,ice_mask, iceObj.sizeX,iceObj.sizeY);
    iceAnimation();
  }
  if (firing) {
    canvas->drawRGBBitmap(fires[fire_counter].posX,fires[fire_counter].posY ,fires[fire_counter].fire,fires[fire_counter].fire_mask, fires[fire_counter].sizeX,fires[fire_counter].sizeY);
    fireAnimation();
  }


  // animation for obstacles
  delay(40);
  if (lives > 0){
    // updates the postition of the obstacles to be drawn
    updateObstacles();
    // draws the obstacles in the correct positions
    for (int i=0; i<max_obstacles; i++) {
      if (obstacles[i].chosen && obstacles[i].power>0) {
        // obstacles are a box with a darker border and an x in the middle
        canvas->fillRect(obstacles[i].posX,obstacles[i].posY, obstacles[i].size,obstacles[i].size, obstacles[i].color);
        canvas->drawRect(obstacles[i].posX, obstacles[i].posY, obstacles[i].size, obstacles[i].size, obstacles[i].borderColor);
        canvas->drawLine(obstacles[i].posX, obstacles[i].posY, obstacles[i].posX+obstacles[i].size, obstacles[i].posY+obstacles[i].size, obstacles[i].borderColor);
        canvas->drawLine(obstacles[i].posX+obstacles[i].size, obstacles[i].posY, obstacles[i].posX, obstacles[i].posY+obstacles[i].size, obstacles[i].borderColor);
       }
    }
  }
  // draws the correct number of lives according to the global variable lives
  handleLives();


  //displaying the elements to pick up
  if(level == 1){
    for(int i = 0; i<max_PU; i++){
      if(PUelements[i].type == 0 && !PUelements[i].picked){
        canvas->drawCircle(PUelements[i].posX, PUelements[i].posY, PUelements[i].size, PUelements[i].color);
      }
      
      if(PUelements[i].type == 1 && !PUelements[i].picked){
        canvas->drawCircle(PUelements[i].posX, PUelements[i].posY, PUelements[i].size, PUelements[i].color);
      }
    }
    updatePU();
  }


  // playing music
  if(musicPlaying) {
    if(arcada.WavReadyForData()) {
      wavStatus status = arcada.WavReadFile();
        // plays the music on loop
        if(status==WAV_EOF) {
          loadMusic();
        }
      }
    }


}



/**
 * draws the correct number of lives according to the global variable 'lives'
 * draws the power gauges in hard mode
 * keeps track of the score in hard mode
 */
void handleLives(){
  GFXcanvas16 *canvas = arcada.getCanvas();
  if (lives==3) {
    canvas->drawRGBBitmap(130,10, life,life_mask, 8,8);
    canvas->drawRGBBitmap(140,10, life,life_mask, 8,8);
    canvas->drawRGBBitmap(150,10, life,life_mask, 8,8);
  }
  else if (lives==2) {
    canvas->drawRGBBitmap(130,10, life,life_mask, 8,8);
    canvas->drawRGBBitmap(140,10, life,life_mask, 8,8);
    canvas->drawRGBBitmap(150,10, nolife,life_mask, 8,8);
  } 
  else if (lives==1) {
    canvas->drawRGBBitmap(130,10, life,life_mask, 8,8);
    canvas->drawRGBBitmap(140,10, nolife,life_mask, 8,8);
    canvas->drawRGBBitmap(150,10, nolife,life_mask, 8,8);
  } else {
    inSelection=true;
    score=0;
    lives = 3;
    icep.power=5;
    firep.power=5;
  }
  // handling powers
  if(level==1) {
    canvas->drawRect(10,60,10,20,ARCADA_RED);
    canvas->drawRect(30,60,10,20,ARCADA_BLUE);
    canvas->fillRect(10, 80-(firep.power*2), 10, firep.power*2, ARCADA_ORANGE);
    canvas->fillRect(30, 80-(icep.power*2), 10, icep.power*2, ARCADA_BLUE); 
  }
  // current score
  if(level==1) {
    score++;
    if(score>highscore) {
      highscore=score;
    }
    canvas->setTextColor(ARCADA_BLUE);
    canvas->setTextSize(1);
    canvas->setCursor(120, 30);
    canvas->print(score);
    // high score
    canvas->setTextColor(ARCADA_BLACK);
    canvas->setTextSize(1);
    canvas->setCursor(120, 20);
    canvas->print(highscore);
  }
  arcada.blitFrameBuffer(0, 0, true, false);
  yield(); 
}





/**
 * updates and resets the values used for the ice power
 */
void iceAnimation() {
  if (iceObj.posY >= 60) {
    switch (iceObj.lane) {
      case 0:
        iceObj.posX = iceObj.posX+5;
      break;
      case 1:
        iceObj.posX = iceObj.posX;
      break;
      case 2:
        iceObj.posX = iceObj.posX-5;
      break;
    }
    iceObj.posY = iceObj.posY-5;
    iceObj.sizeX = iceObj.sizeX-2;
    iceObj.sizeY = iceObj.sizeY-4;
  } else {
    iceObj.posY = 100;
    iceObj.sizeX = 8;
    iceObj.sizeY = 16;
    icing = false;
  }

  // destroys the obstacle according to used power
  handlePowers(0);
}


/**
 * updates and resets the fires array used for the fire power
 */
void fireAnimation() {
  if (fire_counter < 5) {
    fire_counter++;
  } else {
    fires[0].posY = 91;
    fires[5].posY = 91;
    fires[1].posY = 88;
    fires[2].posY = 80;
    fires[3].posY = 80;
    fires[4].posY = 80;
    fire_counter = 0;
    firing = false;
  }

  //destroys the obstacle according to used power
  handlePowers(1);
}


/**
 * decreases obstacle powers according to obstacle type and used powers
 */
void handlePowers(int power) {
  switch(power) {
    case 0: // if using ice powers
      for(int i=0; i<max_obstacles; i++){
        if(obstacles[i].chosen && (obstacles[i].lane == lane) && obstacles[i].obstacleType == 2){   // ice powers can decrease hot obstacles only
          if(iceObj.posY<=obstacles[i].posY){
            // power of obstacles decreases, if it has 0 power, it is destroyed
            obstacles[i].power--;
            if(obstacles[i].power < 0){
              obstacles[i].chosen = false;
              if(level == 1){
                powerUpdate(obstacles[i], 0);
              }
              // setting the power levels of the obstacle new according to easy or hard mode
              switch(level) {
              case 0: obstacles[i].power = 1; break;
              case 1: obstacles[i].power = random(5); break;
              }
            }
          }
        }
      }
    break;
    case 1: // if using fire powers
      for(int i=0; i<max_obstacles; i++){
        if(obstacles[i].chosen && (obstacles[i].lane == lane) && obstacles[i].obstacleType == 1){   // fire powers can decrease cold obstacles only
          if(fires[2].posY<=obstacles[i].posY){
            // power of obstacles decreases, if it has 0 power, it is destroyed
            obstacles[i].power--;
            Serial.println(obstacles[i].power);
            if(obstacles[i].power < 0){
              obstacles[i].chosen = false;
              if(level == 1){
                powerUpdate(obstacles[i], 1);
              }
              // setting the power levels of the obstacle new according to easy or hard mode
              switch(level) {
              case 0: obstacles[i].power = 1; break;
              case 1: obstacles[i].power = random(5); break;
              }
            }
          }
        }
      }
    break;
  }
}



/**
 * handles the button inputs every game loop
 */
void handleButtons() {
  uint8_t buttonState = arcada.readButtons();
  if (buttonState & ARCADA_BUTTONMASK_LEFT) {
    buttonLeft = true;
  }
  if (buttonState & ARCADA_BUTTONMASK_SELECT) {
    inSelection = true;
  }
  if (buttonState & ARCADA_BUTTONMASK_A) {
    if(icep.power > 0){
      icing = true;
      iceObj.lane = lane;
      iceObj.posX = x_pos;
      if (level==1) {
        icep.power -= 1;
        if(icep.power < 0){
          icep.power = 0;
        }
      }
    }
  }
  if (buttonState & ARCADA_BUTTONMASK_B) {
    if(firep.power > 0){
      firing = true;
      for (int i=0; i<6; i++) {
        fires[i].lane = lane;
        fires[i].posX = x_pos;
      }
      if (level==1) {
        firep.power -= 1;
        if(firep.power < 0){
          firep.power = 0;
        }
      }
    }
  }
  if (buttonState & ARCADA_BUTTONMASK_UP){
    jumping = true;
  }
  if (buttonState & ARCADA_BUTTONMASK_DOWN){
    sliding = true;
  }
}


/**
 * initializes the objects to pick up for recharging powers in hard mode
 */
 void initPU(){
  for(int i = 0; i<max_PU; i++){
    if(PUelements[i].posY >= 150){
      PUelements[i].lane = random(3);
      PUelements[i].posY = 60;
      PUelements[i].size = 1;
      PUelements[i].type = random(2);
      switch(PUelements[i].type){
        case 0: PUelements[i].color = icep.color; break;
        case 1: PUelements[i].color = firep.color; break;
      }
      switch(PUelements[i].lane){
        case 0: PUelements[i].posX = 73; break;
        case 1: PUelements[i].posX = 78.5; break;
        case 2: PUelements[i].posX = 82; break;
      }
      PUelements[i].picked = false;
    }
  }
}


/**
 *  animation of the objects to pick up for recharging powers in hard mode
 *  also handles the interaction with the runner
 */
void updatePU(){
  for(int i = 0; i<max_PU; i++){
    switch(PUelements[i].lane){
      case 0:
        PUelements[i].posX = PUelements[i].posX - 3;
        break;
      case 1:
        PUelements[i].posX = PUelements[i].posX + 0.5;
        break;
      case 2:
        PUelements[i].posX = PUelements[i].posX + 4;
        break;
    }
    PUelements[i].posY = PUelements[i].posY + 4;
    PUelements[i].size = PUelements[i].size + 1;

    if(!PUelements[i].picked && !jumping && !sliding){
      if(PUelements[i].lane == lane){
        if(PUelements[i].posY <= y_pos && PUelements[i].posY + PUelements[i].size >= y_pos){
          if(PUelements[i].type == 0){
            icep.power += PUelements[i].power;
            if(icep.power > icep.max){
              icep.power = icep.max; 
            }
            PUelements[i].picked = true;
          } else {
            firep.power += PUelements[i].power;
            if(firep.power > firep.max){
              firep.power = firep.max; 
            }
            PUelements[i].picked = true;
          }
        }
      }
    }
  }
}



/**
 * used for switching between the left and right leg and bouncing of the runner
 */
void legs() {
  if(leg_counter<6) {
    if(leg_counter<3) {
      y_pos = y_pos+2;
    } else {
      y_pos = y_pos-2;
    }
    runner = runner_l;
    runner_mask = runner_l_mask;
    leg_counter = leg_counter+1;
  } else if (leg_counter>=6 && leg_counter<12){
    if(leg_counter==6) {
      y_pos = ARCADA_TFT_HEIGHT-23;
    }
    if(leg_counter<9){
      y_pos = y_pos+2;
    } else {
      y_pos = y_pos-2;
    }
    runner = runner_r;
    runner_mask = runner_r_mask;
    leg_counter = leg_counter+1;
  } else {
    leg_counter = 0;
  }
}




/**
 * puts new obstacles on the playing field
 * the lane, the type, whether it will appear (bool chosen) are decided randomly
 * obstacle types are: 0 normal, 1 cold, 2 hot
 */
void spawnObstacles() {
  currentMillis = millis();
  // resetting the values for obstacles that already passed
  if (currentMillis-previousMillis > 2000) {
    for (int i=0; i<max_obstacles; i++) {
      if (obstacles[i].posY >= 150) {
        obstacles[i].lane = random(3);
        obstacles[i].posY = 60;
        obstacles[i].size = 5;
        obstacles[i].obstacleType = random(3);
        switch (obstacles[i].lane) {
          case 0: obstacles[i].posX = 73; break;
          case 1: obstacles[i].posX = 78.5; break;
          case 2: obstacles[i].posX = 82; break;
        }
        switch (random(2)) {
          case 0: obstacles[i].chosen = true; break;
          case 1: obstacles[i].chosen = false; break;
        }
        switch(obstacles[i].obstacleType) {
          case 0:
            obstacles[i].color = NORMAL_COLOR;
            obstacles[i].borderColor = NORMAL_BORDER;
          break;
          case 1:
            obstacles[i].color = COLD_COLOR;
            obstacles[i].borderColor = COLD_BORDER;
          break;
          case 2:
            obstacles[i].color = HOT_COLOR;
            obstacles[i].borderColor = HOT_BORDER;
          break;
        }
      }
    }
    previousMillis = millis();
  }
}


/**
 * updates the coordinates of the sun and the cloud
 */
void updateSunCloud() {
  // cloud animation
  if(c.posX<160 && c.toRight) {
    c.posX++;
  }
  if(c.posX>=160 && c.toRight) {
    c.toRight=false;
  }
  if(c.posX>10 && !c.toRight) {
    c.posX--;
  }
  if(c.posX<=10 && !c.toRight) {
    c.toRight=true;
  }

  // sun animation
  if(lives>=2) {
    s.faceMask=sunFace_mask;
  } else {
    // the sun will look surprised if you run low on lives
    s.faceMask=sunSurprised_mask;
  }
  if(s.beamCounter>=0 && s.beamCounter<2) {
    s.beamCounter++;
    s.sunMask=sun_mask;
  } else if(s.beamCounter>=2 && s.beamCounter<4) {
    s.beamCounter++;
    s.sunMask=sun_mask2;
  } else {
    s.beamCounter=0;
  }
}


/**
 * animates the blinking effect when hit by an obstacle
 */
void justLostALife() {
  GFXcanvas16 *canvas = arcada.getCanvas();
  if(blinking<=8 && blinking>6) {
    canvas->drawRGBBitmap(handleLane(), y_pos+9, runner,runner_dead_mask, 11,20);
    blinking--;
  } else if(blinking<=6 && blinking>4) {
    blinking--;
  } else if(blinking<=4 && blinking>2) {
    canvas->drawRGBBitmap(handleLane(), y_pos+9, runner,runner_dead_mask, 11,20);
    blinking--;
  } else if(blinking<=2 && blinking>0) {
    blinking--;
  }
}



/**
 * loads the music (bgm.wav) from the flash
 */
void loadMusic() {
  bgm = arcada.openFileByIndex("/music", 0, O_READ, "wav");
  if(!bgm) {
    errorReporter("Fatal error: No sound file present");
  }
  uint32_t sampleRate;
  
  wavStatus status = arcada.WavLoad(bgm, &sampleRate);
  if((status==WAV_LOAD) || (status==WAV_EOF)) {
    arcada.enableSpeaker(true);
    arcada.timerCallback(sampleRate, timedInterruptCallback);
  } else {
    errorReporter("Fatal error: Sound file corrupted.");
  }
}

void timedInterruptCallback(void) {
  wavStatus status = arcada.WavPlayNextSample();
  if (status==WAV_EOF) {
    arcada.timerStop();
    arcada.enableSpeaker(false);
    musicPlaying = false;
    bgm.close();
  } else {
    musicPlaying = true;
  }
}


void errorReporter(const char *error) {
  Serial.begin(9600);
  Serial.println(error);

  // busy wait to keep USB connections, etc. running
  while (true) {
    yield();
  }
}
